-- ----------------------------------------------------------------
-- lib_counter.vhd
--
-- 9/28/2010 D. W. Hawkins (dwh@ovro.caltech.edu)
--
-- Generic counter component.
--
-- ----------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

-- Generic components library
-- * the private components are compiled into this library
library lib_generic;

-- ----------------------------------------------------------------

entity lib_counter is
	generic (
		-- Implementation selection
		LIB_VENDOR    : string  := "ALTERA";
		LIB_TYPE      : string  := "GENERIC";
		LIB_FAMILY    : string  := "DEFAULT";

		-- Component parameters
		LIB_WIDTH     : integer := 8;
		LIB_DIRECTION : string  := "UP"  -- "UP" or "DOWN"
	);
	port (
		rst   : in  std_logic;
		clk   : in  std_logic;
		clr   : in  std_logic;
		load  : in  std_logic;
		en    : in  std_logic;
		d     : in  std_logic_vector(LIB_WIDTH-1 downto 0);
		q     : out std_logic_vector(LIB_WIDTH-1 downto 0);
		cout  : out std_logic
	);
end entity;

-- ----------------------------------------------------------------

architecture top of lib_counter is

	-- ------------------------------------------------------------
	-- Components ('private' implementations)
	-- ------------------------------------------------------------
	--
	-- Generic counter
	component lib_counter_generic is
		generic (
			LIB_WIDTH     : integer;
			LIB_DIRECTION : string
		);
		port (
			rst   : in  std_logic;
			clk   : in  std_logic;
			clr   : in  std_logic;
			load  : in  std_logic;
			en    : in  std_logic;
			d     : in  std_logic_vector(LIB_WIDTH-1 downto 0);
			q     : out std_logic_vector(LIB_WIDTH-1 downto 0);
			cout  : out std_logic
		);
	end component;

	-- Altera LPM_COUNTER based counter
	component lib_counter_altera is
		generic (
			LIB_TYPE      : string;
			LIB_FAMILY    : string;
			LIB_WIDTH     : integer;
			LIB_DIRECTION : string
		);
		port (
			rst   : in  std_logic;
			clk   : in  std_logic;
			clr   : in  std_logic;
			load  : in  std_logic;
			en    : in  std_logic;
			d     : in  std_logic_vector(LIB_WIDTH-1 downto 0);
			q     : out std_logic_vector(LIB_WIDTH-1 downto 0);
			cout  : out std_logic
		);
	end component;

begin

	g1: if (LIB_VENDOR = "GENERIC") generate
		u1: lib_counter_generic
			generic map (
				LIB_WIDTH     => LIB_WIDTH,
				LIB_DIRECTION => LIB_DIRECTION
			)
			port map (
				rst   => rst,
				clk   => clk,
				clr   => clr,
				load  => load,
				en    => en,
				d     => d,
				q     => q,
				cout  => cout
			);
	end generate;

	g2: if (LIB_VENDOR = "ALTERA") generate
		u1: lib_counter_altera
			generic map (
				LIB_TYPE      => LIB_TYPE,
				LIB_FAMILY    => LIB_FAMILY,
				LIB_WIDTH     => LIB_WIDTH,
				LIB_DIRECTION => LIB_DIRECTION
			)
			port map (
				rst   => rst,
				clk   => clk,
				clr   => clr,
				load  => load,
				en    => en,
				d     => d,
				q     => q,
				cout  => cout
			);
	end generate;

end architecture;