-- ----------------------------------------------------------------
-- lib_counter_generic.vhd
--
-- 9/28/2010 D. W. Hawkins (dwh@ovro.caltech.edu)
--
-- Generic implementation of the counter component.
--
-- ----------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

-- ----------------------------------------------------------------

entity lib_counter_generic is
	generic (
		-- Component parameters
		LIB_WIDTH     : integer;
		LIB_DIRECTION : string
	);
	port (
		rst   : in  std_logic;
		clk   : in  std_logic;
		clr   : in  std_logic;
		load  : in  std_logic;
		en    : in  std_logic;
		d     : in  std_logic_vector(LIB_WIDTH-1 downto 0);
		q     : out std_logic_vector(LIB_WIDTH-1 downto 0);
		cout  : out std_logic
	);
end entity;

-- ----------------------------------------------------------------

architecture hdl of lib_counter_generic is

	signal count      : unsigned(LIB_WIDTH-1 downto 0) := (others => '0');
	signal count_max  : unsigned(LIB_WIDTH-1 downto 0);
	signal count_zero : unsigned(LIB_WIDTH-1 downto 0);
	signal up         : std_logic;

begin

	-- Direction control
	up <= '0' when (LIB_DIRECTION = "DOWN") else '1';

	process(clk, rst)
	begin
		if (rst = '1') then
			count <= (others => '0');
		elsif rising_edge(clk) then
			if (clr = '1') then
				count <= (others => '0');
			elsif (load = '1') then
				count <= unsigned(d);
			elsif (en = '1') then
				if (up = '1') then
					count <= count + 1;
				else
					count <= count - 1;
				end if;
			end if;
		end if;
	end process;

	-- Counter output
	q <= std_logic_vector(count);

	-- Carry out
	count_zero <= (others => '0');
	count_max  <= (others => '1');
	process(count, up, count_zero, count_max)
	begin
		if (up = '1') then
			if (count = count_max) then
				cout <= '1';
			else
				cout <= '0';
			end if;
		else
			if (count = count_zero) then
				cout <= '1';
			else
				cout <= '0';
			end if;
		end if;
	end process;

end architecture;