-- ----------------------------------------------------------------
-- bemicro_sdk_cfg.vhd
--
-- 7/02/2011 D. W. Hawkins (dwh@ovro.caltech.edu)
--
-- Arrow BeMicro-SDK Cyclone IV FPGA USB stick configuration.
--
-- This file contains configuration options for the blink LEDs
-- counter. The configuration for the top-level design, i.e.,
-- bemicro_sdk_cfg can be edited to select either the Altera
-- or generic implementation.
--
-- After Quartus completes place-and-route, use the hierachy GUI
-- to view the design components. For the Altera counter, the
-- hierachy is
--
-- - bemicro_sdk
--   - blink_leds:u1
--     - counter_altera:u1
--       - lpm_counter:u1
--
-- while for the generic counter it becomes
--
-- - bemicro_sdk
--   - blink_leds:u1
--     - counter_generic:u1
--
-- ----------------------------------------------------------------

configuration blink_leds_altera_cfg of blink_leds is
	for mixed
		for u1 : up_counter
			--
			-- Map the up_counter to an Altera counter and
			-- connect the unused generics and ports.
			--
			use entity work.counter_altera(structure)
				generic map (
					WIDTH     => COUNT_WIDTH,
					DIRECTION => "UP"
				)
				port map (
					rst   => rst,
					clk   => clk,
					clr   => '0',
					load  => '0',
					en    => '1',
					d     => (others => '0'),
					q     => count
				);

		end for;
	end for;
end configuration;

configuration blink_leds_generic_cfg of blink_leds is
	for mixed
		for u1 : up_counter
			--
			-- Map the up_counter to a generic counter and
			-- connect the unused generics and ports.
			--
			use entity work.counter_generic(hdl)
				generic map (
					WIDTH     => COUNT_WIDTH,
					DIRECTION => "UP"
				)
				port map (
					rst   => rst,
					clk   => clk,
					clr   => '0',
					load  => '0',
					en    => '1',
					d     => (others => '0'),
					q     => count
				);

		end for;
	end for;
end configuration;

configuration example3_cfg of bemicro_sdk is
	for example3
		for u1 : blink_leds
			use configuration work.blink_leds_altera_cfg;
--			use configuration work.blink_leds_generic_cfg;
		end for;
	end for;
end configuration;